unit sysinfo;
{
System info component
---------------------
It shows following information:
- Register information of operating system (Owner, Organisation)
- Windows type, version, build, install date, path and temporary path
- network computer name and user name logged on
- cpu type and free space on boot drive
- physical memory available, total, usage
- swap file setting, current size, usage
- free system, gdi and user resources
All information is available at design time.
Suggestions/improvements welcome.
-----------------------------------------------------------------------
Quality freeware from Sight&Sound, Slovenia : http://www.sight-sound.si
Version 1.0: release date : 23.11.1996
Version 1.1: release date : 24.11.1996
- bug fix at memory usage calculation
}

interface

uses
 SysUtils, Windows, Classes, Registry;

type
   TSysInfo = class(TComponent)
   private
      FRegOrganisation : string;
      FRegOwner : string;
      FCPUType : string;
      FWinType : string;
      FWinVersion : string;
      FWinBuild : string;
      FMemTotal : string;
      FMemAvailable : string;
      FMemUsage : string;
      FDriveC : string;
      FWinInstallDate : string;
      FWinPath : string;
      FWinTempPath : string;
      FNetComputerName : string;
      FNetUserName : string;
      FSwapFileSetting : string;
      FSwapFileSize : string;
      FSwapFileUsage : string;
      FFreeSystemRes : string;
      FFreeGDIRes : string;
      FFreeUserRes : string;
      procedure NoModify(value:string);
   protected
   public
      constructor Create(AOwner:TComponent);override;
      procedure Refresh;
   published
      Property RegOrganisation : string read FRegOrganisation write NoModify stored false;
      Property RegOwner : string read FRegOwner write NoModify stored false;
      Property CPUType : string read FCPUType write NoModify stored false;
      Property WinType : string read FWinType write NoModify stored false;
      Property WinVersion : string read FWinVersion write NoModify stored false;
      Property WinBuild : string read FWinBuild write NoModify stored false;
      Property MemAvailable : string read FMemAvailable write NoModify stored false;
      Property MemTotal : string read FMemTotal write NoModify stored false;
      Property MemUsage : string read FMemUsage write NoModify stored false;
      Property DriveC : string read FDriveC write NoModify stored false;
      Property WinInstallDate : string read FWinInstallDate write NoModify stored false;
      Property WinPath : string read FWinPath write NoModify stored false;
      Property WinTempPath : string read FWinTempPath write NoModify stored false;
      Property NetComputerName : string read FNetComputerName write NoModify stored false;
      Property NetUserName : string read FNetUserName write NoModify stored false;
      Property SwapFileSetting : string read FSwapFileSetting write NoModify stored false;
      Property SwapFileSize : string read FSwapFileSize write NoModify stored false;
      Property SwapFileUsage : string read FSwapFileUsage write NoModify stored false;
      Property FreeSystemRes : string read FFreeSystemRes write NoModify stored false;
      Property FreeGDIRes : string read FFreeGDIRes write NoModify stored false;
      Property FreeUserRes : string read FFreeUserRes write NoModify stored false;
      Property Name;
end;

procedure Register;

implementation

const
  Gfsr_SystemResources = 0;
  Gfsr_GdiResources = 1;
  Gfsr_UserResources = 2;

var
  hInst16: THandle;
  GFSR: Pointer;

{ Undocumented Kernel32 calls. }
function LoadLibrary16(LibraryName: PChar): THandle; stdcall; external kernel32 index 35;
procedure FreeLibrary16(HInstance: THandle); stdcall; external kernel32 index 36;
function GetProcAddress16(Hinstance: THandle; ProcName: PChar): Pointer; stdcall; external kernel32 index 37;
procedure QT_Thunk; cdecl; external kernel32 name 'QT_Thunk';

{ QT_Thunk needs a stack frame. }
{$StackFrames On}

{ Thunking call to 16-bit USER.EXE. The ThunkTrash argument
  allocates space on the stack for QT_Thunk. }
function NewGetFreeSystemResources(SysResource: Word): Word;
var
  ThunkTrash: array[0..$20] of Word;
begin
  { Prevent the optimizer from getting rid of ThunkTrash. }
  ThunkTrash[0] := hInst16;
  hInst16 := LoadLibrary16('user.exe');
  if hInst16 < 32 then
    raise Exception.Create('Can''t load USER.EXE!');
  { Decrement the usage count. This doesn't really free the
    library, since USER.EXE is always loaded. }
  FreeLibrary16(hInst16);
  { Get the function pointer for the 16-bit function in USER.EXE. }
  GFSR := GetProcAddress16(hInst16, 'GetFreeSystemResources');
  if GFSR = nil then
    raise Exception.Create('Can''t get address of GetFreeSystemResources!');
  { Thunk down to USER.EXE. }
  asm
    push SysResource  { push arguments }
    mov edx, GFSR     { load 16-bit procedure pointer }
    call QT_Thunk     { call thunk }
    mov Result, ax    { save the result }
  end;
end;

procedure TSysInfo.NoModify(value:string);
begin
  raise Exception.Create('Can''t modify a read-only property');
end;

procedure TSysInfo.Refresh;
var
 RegIni  : TRegIniFile;
 SI      : TSystemInfo;
 OS      : TOSVersionInfo;
 MemStat : TMemoryStatus;
 di      : longint;
 buf     : array [0..3] of byte;
 PathArray : array [0..255] of char;
begin
 RegIni := TRegIniFile.Create('');
 with RegIni do
  begin
   RootKey := HKEY_LOCAL_MACHINE;
   LazyWrite := True;
   OpenKey('\SOFTWARE\Microsoft\Windows',False);
   FRegOrganisation := ReadString('CurrentVersion', 'RegisteredOrganization', 'unknown');
   FRegOwner := ReadString('CurrentVersion', 'RegisteredOwner', 'unknown');
   OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion',False);
   di := readbinarydata ('FirstInstallDateTime',buf,sizeof(buf));
   FWinInstallDate := DateTimeToStr(FileDateToDateTime(buf[0]+buf[1]*256+buf[2]*65535+buf[3]*16777216));
   Free;
  end;
 GetSystemInfo(SI);
 case Si.dwProcessorType of
  386 : FCPUType := 'Intel 386';
  486 : FCPUType := 'Intel 486';
  586 : FCPUType := 'Intel Pentium';
 end;
 OS.dwOSVersionInfoSize := sizeof(TOSVERSIONINFO);
 GetVersionEx(OS);
 case OS.dwPlatformId of
  VER_PLATFORM_WIN32s        : FWinType := 'Windows 3.1x/32s';
  VER_PLATFORM_WIN32_WINDOWS : FWinType := 'Windows 95';
  VER_PLATFORM_WIN32_NT	     : FWinType := 'Windows NT';
 end;
 with OS do
  begin
   FWinVersion := format ('%d.%d', [dwMajorVersion,dwMinorVersion]);
   FWinBuild      := format('%d', [LOWORD(dwBuildNumber)]);
  end;
 MemStat.dwLength := sizeof(TMemoryStatus);
 GlobalMemoryStatus(MemStat);
 with MemStat do
 begin
  FMemTotal     := Format('%d KB',[Trunc(dwTotalPhys/1024)]);
  FMemAvailable := Format('%d KB',[Trunc(dwAvailPhys/1024)]);
  FMemUsage     := Format('%d %%',[trunc(dwAvailPhys/dwTotalPhys*100)]);
  FSwapFileSetting := Format('%d KB',[Trunc(dwTotalPageFile/1024)]);
  FSwapFileSize    := Format('%d KB',[Trunc((dwTotalPageFile-dwAvailPageFile)/1024)]);
  FSwapFileUsage   := Format('%d %%',[100-trunc(dwAvailPageFile/dwTotalPageFile*100)]);
 end;
 di := DiskSize(3);
 if di <> -1 then
  FDriveC := Format('%d MB', [di div 1048576])
 else
  FDriveC := '';

 FillChar(PathArray, SizeOf(PathArray), #0);
 GetWindowsDirectory(PathArray,255);
 FWinPath := Format('%s',[PathArray]);

 FillChar(PathArray, SizeOf(PathArray), #0);
 ExpandEnvironmentStrings('%TEMP%', PathArray, 255);
 FWinTempPath := Format('%s',[PathArray]);

 FillChar(PathArray, SizeOf(PathArray), #0);
 di := 255;
 GetComputerName(PathArray,dword(di));
 FNetComputerName := Format('%s',[PathArray]);

 FillChar(PathArray, SizeOf(PathArray), #0);
 di := 255;
 GetUserName(PathArray,dword(di));
 FNetUserName := Format('%s',[PathArray]);
 FFreeSystemRes := Format('%d %%',[NewGetFreeSystemResources(Gfsr_SystemResources)]);
 FFreeGDIRes := Format('%d %%',[NewGetFreeSystemResources(Gfsr_GDIResources)]);
 FFreeUserRes := Format('%d %%',[NewGetFreeSystemResources(Gfsr_UserResources)]);
end;

constructor TSysInfo.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 Refresh;
end;

procedure Register;
begin
 RegisterComponents('Samples', [TSysInfo]);
end;

end.
